/*

    Meta-Environment - An environment for language prototyping.
    Copyright (C) 2000  Stichting Mathematisch Centrum, Amsterdam, The Netherlands. 

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

    $Id: SeTool.java,v 1.17 2000/04/17 15:27:27 markvdb Exp $
*/
// $Id: SeTool.java,v 1.17 2000/04/17 15:27:27 markvdb Exp $

package nl.cwi.gipe.stred;
import toolbus.aterm.*;
import toolbus.tool.*;
import java.io.*;
import java.net.*;
import java.util.*;

public class SeTool extends Se {

    /** Activates printing of debugging statements */
    private static final boolean debugOn = false;
    private static final String version = "0.1";



  private static Hashtable editors = new Hashtable();
    //  private static ATerm tree = new ATerms(ATerm.the_world);
    //  private static FocusList fl = new FocusList();

  public SeTool(String args[]) throws Exception {
    super(args);
  }
 
  public ATerm getTree(String fid) throws ToolException {
      ATermPattern p;
      ATerm tree;
      ATerm result;
      Editor e = (Editor) editors.get(fid);
      try {
	  p = new ATermPattern(ATerm.the_world,"snd-value(<term>)");
      }
      catch (Exception ex) {
	  throw new ToolException(this,"parse error");
      }
      // Editor.tree() returns a list with one element: the tree. This is
      // because it makes traversal easier, but shouldn't be visible
      // on the outside. So we return the actual tree:
      tree = ((ATermList) e.tree()).getATerms().getFirst();
      result = p.make(tree);
      return result;
  }

  public ATerm getFocus(String fid, int i1) throws ToolException {
    ATermPattern p;
    //    i1--;
    try {
      p = new ATermPattern(ATerm.the_world,"snd-value(<term>)");
    }
    catch (Exception e) {
      throw new ToolException(this,"parse error");
    }
    Editor e = (Editor) editors.get(fid);
    try {
      return p.make(StructureEdit.getFocus(e,i1));
    }
    catch (ParseError se) {
	throw new ToolException(this,"Parse error in insertChars: "+se.getMessage());
    }
  }

  public ATerm replaceFocus(String fid, ATerm t1) throws ToolException {
    ATermPattern p;
    //System.err.println("SeTool.replaceFocus: t1: "+t1.toString());
    try {
      p = new ATermPattern(ATerm.the_world,"snd-value(<term>)");
    }
    catch (Exception ex) {
      throw new ToolException(this,"parse error");
    }
    Editor e = (Editor) editors.get(fid);
    e = StructureEdit.replaceFocus(e,t1);
    editors.put(fid,e);
    FocusList fl = e.focuses();
    return p.make(fl.toATerm());
  }

  public ATerm newEditor(String fid, String s0) throws ToolException {
      //    s0 = killQuotes(s0);
      //    System.err.println(s0);
    ATermPattern p = null;
    try {
      p = new ATermPattern(ATerm.the_world,"snd-value(<term>)");
    }
    catch (Exception e){
      throw new ToolException(this,"parse error");
    }
    Editor e = StructureEdit.newEditor(s0);
    editors.put(fid,e);
    FocusList fl = e.focuses();
    ATerm t = p.make(fl.toATerm());
    //    System.err.print(t.toString());
    return t;
  }
  
  public ATerm insertChars(String fid, int i1, String s2) throws ToolException {
      //    i1--;
    //    System.err.println(s2);
    ATermPattern p;
    try {
      p = new ATermPattern(ATerm.the_world,"snd-value(<term>)");
    }
    catch (Exception e){
       throw new ToolException(this,"parse error");
    }  
    Editor e = (Editor) editors.get(fid);
    try {
      e = StructureEdit.insertChars(e,i1,s2);
    }
    catch (SubtreeException se) {
	throw new ToolException(this,"SubtreeException in insertChars: "+se.getMessage());
    }
    catch (ParseError se) {
	throw new ToolException(this,"Parse error in insertChars: "+se.getMessage());
    }
    editors.put(fid,e);
    FocusList fl = e.focuses();
    
    //System.err.println("SeTool.insertChars: fl: "+fl.toString());
    return p.make(fl.toATerm());
  }

  public ATerm deleteChars(String fid,int i1) throws ToolException {
    ATermPattern p;
    //    i1--;
    try {
      p = new ATermPattern(ATerm.the_world,"snd-value(<term>)");
      //    Editor e = (Editor) a0;
      Editor e = (Editor) editors.get(fid);
      e = StructureEdit.deleteChars(e,i1);
      editors.put(fid,e);
      FocusList fl = e.focuses();
      return p.make(fl.toATerm());
    }
    catch (ParseError se) {
	throw new ToolException(this,"Parse error in deleteChars: "+se.getMessage());
    }

  }

  public ATerm nextFocus(String fid) throws ToolException {
    Editor e = (Editor) editors.get(fid);
    FocusList fl = e.focuses();
    fl.tail();
    e = e.setFocuses(fl);
    editors.put(fid,e);
    try {
      ATermPattern p = new ATermPattern(ATerm.the_world,"snd-value(<term>)");
    return p.make(fl.toATerm());
    }
    catch (ParseError pe) {
      throw new ToolException(this,"parse error");
    }
  }

    /** Replace the focuslist of an editor with a new one
	@param fid The file-id of the editor you want to modify
	@param l The focuslist you want to insert. 
	@return The replaced focuslist, as a snd-value(..) term. This method returns its second argument. 
    */
  public ATerm setFocus(String fid,ATermList l) throws ToolException {
    Editor e = (Editor) editors.get(fid);
    FocusList fl = e.focuses();
    fl = new FocusList(l);
    e = e.setFocuses(fl);
    editors.put(fid,e);
    try {
      ATermPattern p = new ATermPattern(ATerm.the_world,"snd-value(<term>)");
    return p.make(fl.toATerm());
    }
    catch (ParseError pe) {
      throw new ToolException(this,"parse error");
    }
  }

    /** Insert a single focus in a focus list.
	@param fid The file-id of the editor you want to modify 
	@param focus The focus that will be inserted.
	@return A focus list with the new focus inserted.
    */
  public ATerm setFocus(String fid,ATerm focus) throws ToolException {
      Focus f = new Focus((ATermAppl) focus);
    Editor e = (Editor) editors.get(fid);
    FocusList fl = e.focuses();
    fl.insertHead(f);
    e = e.setFocuses(fl);
    editors.put(fid,e);
    try {
      ATermPattern p = new ATermPattern(ATerm.the_world,"snd-value(<term>)");
    return p.make(fl.toATerm());
    }
    catch (ParseError pe) {
      throw new ToolException(this,"parse error");
    }
  }

    public ATerm moveFocus(String fid, String move) throws ToolException {
	Editor e = (Editor) editors.get(fid);
	try {
	    if (move.equals("Left")) {
		e = StructureEdit.moveFocus(e,new Move(Move.LEFT));
	    } else if (move.equals("Right")) {
		e = StructureEdit.moveFocus(e,new Move(Move.RIGHT));
	    } else if (move.equals("Up")) {
		e = StructureEdit.moveFocus(e,new Move(Move.UP));
	    } else if (move.equals("Down")) {
		e = StructureEdit.moveFocus(e,new Move(Move.DOWN));
	    } else {
		throw new ToolException(this,"Illegal move requested: "+move);
	    }
	}
	catch (MoveException me) {
	    throw new ToolException(this,"Exception in focus move");
	}
	FocusList fl = e.focuses();
	try {
	    ATermPattern p = new ATermPattern(ATerm.the_world,"snd-value(<term>)");
	    return p.make(fl.toATerm());
	}
	catch (ParseError pe) {
	    throw new ToolException(this,"parse error");
	}
    }

  public ATerm makeTreeInvalid(String fid) throws ToolException {
      ATermPattern p;
      try {
	  p = new ATermPattern(ATerm.the_world,"snd-value(<term>)");
      }
      catch (Exception e){
	  throw new ToolException(this,"parse error");
      }  
     
      Editor e = (Editor) editors.get(fid);

      e = StructureEdit.makeTreeInvalid(e);
      editors.put(fid,e);

      return p.make(e.focuses().toATerm());
  }					      

  public void recTerminate(ATerm t0) {
    System.exit(0);
  }

  private static String killQuotes(String s) {
    char tmp[] = new char[s.length()-2];
    s.getChars(1,s.length()-1,tmp,0);
    return new String(tmp);
  }

    /** Prints a string as debug output, but only when the debug field is 
	true */
    public static void debug(String s) {
	if (debugOn) {
	    System.err.println(s);
	}
    }

    public static void main(String argv[]) throws Exception {

	switch(argv.length) {
	case 0:
	    usage();
	    break;
	case 1:
	    /* Dump version number */
	    if(argv[0].equals("-V") || argv[0].equals("--version")) {
		System.err.println("Structure Editor version " + version);
		System.exit(1);
	    } else {
		usage();
	    }
	    break;
	default:
	    /* Normal execution of tool */
	    SeTool se = new SeTool(argv);
	    se.connect();
	    se.run();
	}
    }

    private static void usage() {
	System.err.println("Usage: java nl.cwi.gipe.stred.SeTool [-V] [--version] [toolbus-options]");
	System.exit(1);
    }
}
